import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.Spring;
import javax.swing.SpringLayout;

public class ImagePanel extends JPanel {
    private JLabel label;
    private BufferedImage image;
    private BufferedImage reflection;

    private int width;
    private int height;

    private boolean isReflected;
    private boolean focussing;

    private Color focusColor = new Color(64, 64, 64);

    public ImagePanel(String filename) throws IOException {
        super.setName("thumb");

        setFocusable(true);
        addFocusListener(new FocusListener() {
                public void focusGained(FocusEvent event) {
                    focussing = true;
                    ImagePanel panel = (ImagePanel)event.getComponent();
                    panel.repaint(0, 0, 0,
                                  panel.getWidth(), panel.getHeight());
                }
                public void focusLost(FocusEvent event) {
                    focussing = false;
                    ImagePanel panel = (ImagePanel)event.getComponent();
                    panel.repaint(0, 0, 0,
                                  panel.getWidth(), panel.getHeight());
                }
            });
                                              
        File file = new File(filename);
        image = ImageIO.read(file);
        width = image.getWidth();
        height = image.getHeight();
        reflection = createReflection(image);

        label = new JLabel(file.getName(), JLabel.CENTER);
        add(label);

        SpringLayout layout = new SpringLayout();
        setLayout(layout);

        layout.putConstraint(SpringLayout.WEST, label,
                             Spring.constant(2, 5, 10),
                             SpringLayout.WEST, this);
        layout.putConstraint(SpringLayout.EAST, label,
                             Spring.constant(2, 5, 10),
                             SpringLayout.EAST, this);
        layout.putConstraint(SpringLayout.SOUTH, label,
                             Spring.constant(0),
                             SpringLayout.SOUTH, this);

        Dimension dim 
            = new Dimension(108,
                            108 + label.getPreferredSize().height + 10);
        setPreferredSize(dim);
    }

    public void paintComponent(Graphics g) {
        Graphics2D g2d = (Graphics2D)g;

        if (focussing) {
            g2d.setPaint(focusColor);
            g2d.fillRect(0, 0, getWidth(), getHeight());
        }
        
        int x = (getWidth() - width) / 2;
        int y = 108 - height ;

        g2d.setColor(Color.WHITE);
        g2d.fillRect(x-1, y-1,
                     width + 2, height + 2);

        if (isReflected) {
            g2d.drawImage(reflection, x, y, null);
        } else {
            g2d.drawImage(image, x, y, null);
        }
    }

    public void setReflected(boolean isReflected) {
        this.isReflected = isReflected;
    }

    private BufferedImage createReflection(BufferedImage image) {
        int height = image.getHeight();
        
        BufferedImage result = new BufferedImage(image.getWidth(), height * 2,
                BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = result.createGraphics();
        
         // Paints original image
        g2.drawImage(image, 0, 0, null);
        
        // Paints mirrored image
        g2.scale(1.0, -1.0);
        g2.drawImage(image, 0, -height - height, null);
        g2.scale(1.0, -1.0);

        // Move to the origin of the clone
        g2.translate(0, height);
        
        // Creates the alpha mask
        GradientPaint mask;
        mask = new GradientPaint(0, 0, new Color(1.0f, 1.0f, 1.0f, 0.5f),
                0, height / 3, new Color(1.0f, 1.0f, 1.0f, 0.0f));
        Paint oldPaint = g2.getPaint();
        g2.setPaint(mask);
        
        // Sets the alpha composite
        g2.setComposite(AlphaComposite.DstIn);
        
        // Paints the mask
        g2.fillRect(0, 0, image.getWidth(), height);
 
        g2.dispose();
        return result;
    }
}